<?php
/*
    author: Matheus P. Alves Nogueira
    date: 2017
    e-mail: matheus.nogueira@teknisa.com
*/

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DTO\Response\Message;

use Teknisa\MAN\Services\ConsolidacaoVendasService;

class ConsolidacaoVendas extends CrudWithParameterBag {

    protected $dataSourceName = "";
    protected $parameterBag;
    protected $utilService;
    protected $parameterBagColumns = array();
    protected $codReferencia = '';

    protected $consolidationFileLogService;
    protected $consolidacaoVendasService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, ConsolidacaoVendasService $consolidacaoVendasService, Utilities\Util $utilService ) {
        ini_set('max_execution_time', 1800);
        $this->consolidacaoVendasService = $consolidacaoVendasService;
        $this->consolidationFileLogService = $this->consolidacaoVendasService->getLogService();
        $this->utilService      = $utilService;
        parent::__construct($dataSourceManager, $parameterBag);
    }

    public function find(Request\Filter $request, Response $response){
        try {
            ini_set('max_execution_time', 1800);
    		ini_set('memory_limit', '1G');

            $dataSourceName = $this->dataSourceName = 'vnd05400_consolidacaodevendas';
            $conditions = $this->utilService->getParamsAsArray($request);

            if(array_key_exists('TODASNF', $conditions) && $conditions['TODASNF'] == "") {
                $conditions['TODASNF'] = 'S';
            }

            if(!array_key_exists('DTVENDA', $conditions)) {
                $dataSourceFilter = array();
                $likeableConditions = $this->utilService->getLikeConditions($request);

                foreach ($likeableConditions as $condition) {
        			if(array_key_exists($condition['columnName'], $conditions)) {
        				 unset($conditions[$condition['columnName']]);
        			}
        		}

                $conditions['CDCAIXA'] = implode('_', $conditions['CDCAIXA']);

                if(!empty($conditions['NRSEQVENDA'])){
                    $conditions['NRSEQVENDA'] = implode('_', $conditions['NRSEQVENDA']);
                } else {
                    $conditions['NRSEQVENDA'] = null;
                }

                if(!empty($conditions['CDCLIENTE'])){
                    $conditions['CDCLIENTE'] = implode('_', $conditions['CDCLIENTE']);
                }

                if(!empty($conditions['CDTIPOCONS'])){
                    $conditions['CDTIPOCONS'] = implode('_', $conditions['CDTIPOCONS']);
                } else {
                    $conditions['CDTIPOCONS'] = null;
                }

                if(!empty($conditions['CDCCUSCLIE'])){
                    $conditions['CDCCUSCLIE'] = implode('_', $conditions['CDCCUSCLIE']);
                } else {
                    $conditions['CDTIPOCONS'] = null;
                }

                if(!empty($conditions['CDPRODUTO'])){
                    $conditions['CDPRODUTO'] = implode('_', $conditions['CDPRODUTO']);
                } else {
                    $conditions['CDPRODUTO'] = null;
                }

                $novaData = $this->consolidacaoVendasService->verifyDateProduct($conditions);

                if($novaData != '' && $novaData != null && $novaData < $conditions['DTFINAL']) {
                   throw new \Exception("A consulta ultrapassou o limite de produtos! É possível filtrar até dia ".$novaData);
                }

                $newRequest = $this->createNewZeedhiFilterRequestObjectLocal($request, $this->dataSourceName, $conditions);
                foreach ($likeableConditions as $condition) {
                    $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
        		}

                $this->parameterBagColumns = array("CDFILIAL","CDLOJA","DTINICIAL","DTFINAL","CDCAIXA","TCDCAIXA","CDCLIENTE","TCDCLIENTE","NRSEQVENDA","TODASVND","CDTIPOCONS","TCDTIPOCONS","CDCCUSCLIE","TCDCCUSCLIE","CDPRODUTO","TCDPRODUTO","TODASNF","NRORG");
                parent::find($newRequest, $response);
            }

        } catch(\Exception $e) {
             $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function findVendas(Request\Filter $request, Response $response){
        $dataSourceName            = $this->dataSourceName = 'vnd05400_consolidacaodevendas_nrseqvenda';
        $conditions = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->utilService->getLikeConditions($request);
        foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
			if(array_key_exists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}

		if(!empty($conditions['CDCAIXA'])) {
            $conditions['CDCAIXA'] = implode('_', $conditions['CDCAIXA']);
            $conditions['TCDCAIXA'] = '';
		} else {
            $conditions['CDCAIXA'] = '';
            $conditions['TCDCAIXA'] = 'T';
		}

        $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        foreach ($likeableConditions as $condition) {
                $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
        $this->parameterBagColumns = array("DTFINAL","DTINICIAL","TCDCAIXA","CDCAIXA","CDFILIAL","CDLOJA","NRORG");
        parent::find($newRequest, $response);
    }

    public function findCentroCusto(Request\Filter $request, Response $response){
        $this->dataSourceName = 'vnd05400_consolidacaodevendas_cdccusclie';
        $this->parameterBagColumns = array("CDCLIENTE","TCDCLIENTE","NRORG");

        $conditions = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->utilService->getLikeConditions($request);

        foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
			if(array_key_exists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}

		if(!empty($conditions['CDCLIENTE'])) {
            $conditions['CDCLIENTE'] = implode('_', $conditions['CDCLIENTE']);
            $conditions['TCDCLIENTE'] = '';
		} else {
            $conditions['CDCLIENTE'] = '';
            $conditions['TCDCLIENTE'] = 'T';
		}

        $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        foreach ($likeableConditions as $condition) {
            $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
        parent::find($newRequest, $response);
    }

    public function findProduto(Request\Filter $request, Response $response){
        $this->dataSourceName = 'vnd05400_consolidacaodevendas_cdproduto';
        $this->parameterBagColumns = array(
            "CDFILIAL",
            "CDLOJA",
            "CDCAIXA",
            "TCDCAIXA",
            "DTINICIAL",
            "DTFINAL",
            "NRSEQVENDA",
            "TODASVND",
            "NRORG"
        );
        $conditions = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->utilService->getLikeConditions($request);

        foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
			if(array_key_exists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}

		if(!empty($conditions['CDCAIXA'])) {
            $conditions['CDCAIXA'] = implode('_', $conditions['CDCAIXA']);
            $conditions['TCDCAIXA'] = '';
		} else {
            $conditions['CDCAIXA'] = '';
            $conditions['TCDCAIXA'] = 'T';
		}

		if(!empty($conditions['NRSEQVENDA'])) {
            $conditions['NRSEQVENDA'] = implode('_', $conditions['NRSEQVENDA']);
            $conditions['TODASVND'] = '';
		} else {
            $conditions['NRSEQVENDA'] = '';
            $conditions['TODASVND'] = 'S';
		}

        $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        foreach ($likeableConditions as $condition) {
            $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
        parent::find($newRequest, $response);
    }

    public function mountFind($request, $response, $dataSource){
    // método usado para sobrescrever o CrudWithParameterBag por causa de tratamentos adicionais
    // busca dados dinamicamente
        $filterCriteria = $request->getFilterCriteria();

        $this->consolidacaoVendasService->setParameterBagColumns($this->parameterBagColumns, $filterCriteria->getConditions());
        $this->consolidacaoVendasService->setParameterBagValues($this->parameterBag, $filterCriteria->getConditions());
        $this->consolidacaoVendasService->setDataSourceName($this->dataSourceName, $dataSource);
        $newRequest = $this->consolidacaoVendasService->newFilter($request, $this->dataSourceName);

        parent::find($newRequest, $response);
    }

    public function getParavendParams(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->dataSourceName = 'getParavendParams';

        try {
            $paravendParams = $this->consolidacaoVendasService->getParavendParams($row['CDFILIAL'], $row['NRORG']);
            self::responseFrontEnd($response, true, $paravendParams);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }

    public function dateRestriction(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->dataSourceName = 'dateRestriction';
        $dateRestriction = null;
        try {
            $dateRestriction = $this->consolidacaoVendasService->dateRestriction($row['filterQuery'], $row['selectQuery']);
            self::responseFrontEnd($response, true, $dateRestriction);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }

    public function groupConsolidation(Request\Row $request, Response $response){
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');

        // agrupa consolidação
        $row = $request->getRow();
        $this->dataSourceName = 'groupConsolidation';

        $products     = $row['products'];
        $filterParams = $row['filterParams'];
        $filialParams = $row['filialParams'];
        $lojaParams   = $row['lojaParams'];
        $paramEstoque = array();
        $groupConsolidation = array();
        $this->codReferencia = $filterParams['NRORG'].$filterParams['CDFILIAL'].$filterParams['CDLOJA'];
        try {

            //excluir arquivo Consolidacao
            if(file_exists('/tmp/RotinaConsolidacaoRetail_'.date_create()->format('Ymd').'_'.$this->codReferencia.'_processes.txt')){
                unlink ('/tmp/RotinaConsolidacaoRetail_'.date_create()->format('Ymd').'_'.$this->codReferencia.'_processes.txt');
            }

            //iniciando a consolidacao
            $this->consolidationFileLogService->saveProcessLog('okInitgroupConsolidation', '', $this->codReferencia);
            $paramEstoque = $this->consolidacaoVendasService->getParamEstoque($filterParams['NRORG']);
            $this->consolidationFileLogService->saveProcessLog('okStockParams', '', $this->codReferencia);

            // testa consolidação por data da venda
            $consolidationForSaleDate = $this->consolidacaoVendasService->consolidationForSaleDate($products, $filterParams, $filialParams, $paramEstoque);
            $this->consolidationFileLogService->saveProcessLog('okConsolidationPerDateTest', '', $this->codReferencia);
            if ($consolidationForSaleDate['status']) {
                // reúne produtos da consolidação com as observações
                $this->consolidationFileLogService->saveProcessLog('okProductsDoHaveObs', '', $this->codReferencia);
                $groupConsolidation = $this->consolidacaoVendasService->groupConsolidation($filterParams, $filialParams, $paramEstoque, $lojaParams, $products);
                $this->consolidationFileLogService->saveProcessLog('okGroupedProductsWithObs', '', $this->codReferencia);
                self::responseFrontEnd($response, true, $groupConsolidation);
            } else {
                $this->consolidationFileLogService->saveProcessLog('okProductsDoNotHaveObs', '', $this->codReferencia);
                self::responseFrontEnd($response, true, $consolidationForSaleDate);
            }
        } catch (Exception $e) {
            $this->consolidationFileLogService->saveExceptionLog('exceptionIntro', $e, $this->codReferencia);
            self::responseFrontEnd($response, false, $e);
        }
    }

    public function groupProducts(Request\Row $request, Response $response) {

        ini_set('max_execution_time', 4000);
		ini_set('memory_limit', '4096M');

        // agrupa produtos da consolidacao
        $this->consolidationFileLogService->saveProcessLog('okGroupProducts', '', $this->codReferencia);
        $this->dataSourceName = 'groupConsolidation';
        $row = $request->getRow();
        $groupConsolidation = $this->consolidacaoVendasService->agruparProdutosConsolidacao($row[1], $row[2], $row[3], $row[4]);
        $this->consolidationFileLogService->saveProcessLog('okProductsGroupedWithSuccess', '', $this->codReferencia);
        self::responseFrontEnd($response, true, $groupConsolidation);
    }

    public function insertConsolidation(Request\Row $request, Response $response){

        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');

        // insere consolidação no banco
        $row = $request->getRow();
        $this->dataSourceName = 'insertConsolidation';

        $data         = $row['toConsolidate'];
        $dataEstRel         = $row['toconsolidationEstRel'];

        $gridItens    = $row['gridItens'];
        $filterParams = $row['filterParams'];
        $filialParams = $row['filialParams'];

        if(empty($filterParams['CDSETOR']))
            $filterParams['CDSETOR'] = null;

        $insertConsolidation = array();
        try {
            $insertConsolidation = $this->consolidacaoVendasService->insertConsolidation($data, $gridItens, $filterParams, $filialParams, $dataEstRel);
            self::responseFrontEnd($response, true, $insertConsolidation);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }

    public function responseFrontEnd(&$response, $type, $data) {
        if ($type)
            $response->addDataSet(new DataSet($this->dataSourceName, $data));
        else
            $response->setError(new Response\Error($data->getMessage(), $data->getCode()));
    }

    public function getLikeConditions($request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = array();
        foreach ($conditions as $condition) {
            if($condition['operator'] === 'LIKE_ALL' )
                array_push($result, $condition);
        }
        return $result;
    }

    public function habilitaPesqProd(Request\Row $request, Response $response) {
        $row = $request->getRow();

        $params = array(
            ':CDFILIAL' => $row['CDFILIAL'],
            ':NRORG'    => $row['NRORG']
        );
        if(!empty($row['CDFILIAL']) && !empty($row['NRORG'])){
            $habilitaProd[0] = $this->consolidacaoVendasService->habilitaPesqProd($params)[0]['IDGERANFVENPROD'];
        }else{
            $habilitaProd[0] = 'N';
        }

        $response->addDataSet(new DataSet('habilitaProd', $habilitaProd));
    }


    //funcao copiada do util para adaptacao da tela, sem pagination, retirada a pagination para nao consolidar mais de uma vez os mesmos produtos
    public function createNewZeedhiFilterRequestObjectLocal($request, $dataSourceName, $conditions) {
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }

        return new DTO\Request\Filter(
            new FilterCriteria(
                $dataSourceName,
                $dataSourceFilter,
                null,
                null
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
    }


}