<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services\ParamKDSService;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\MAN\Utilities;

class ParaEmailResp extends \Teknisa\MAN\Controllers\TeknisaCrud{

    const dataSourceProjetosDisponiveis = 'projetosDispResp';
    const dataSourceEmails              = 'paraEmailResp';
    const dataSourceUnidade             = 'unidadeProjeto';
    const dataSourceEmailUnidade        = 'emailUnidadeParticipante';
    const dataSourceSubProjetos         = 'subProjetos';
    const dataSourceEmailSubProjeto     = 'emailSubProjeto';
    const dataSourceUnidadeSubProjeto   = 'unidadeSubProjeto';
    const dataSourceEmailUnidadeParticipanteSubProjeto = 'emailUnidadeParticipanteSubProjeto';
    const dataSourceCodExterno          = 'paraCodExterno';
    const dataSourceCodExternoUnidade   = 'paraCodExternoUnidade';
    const dataSourceCodExternoSubProjeto = 'paraCodExternoSubProjeto';
    const dataSourceCodExternoUnidadeSubProjeto = 'paraCodExternoUnidadeSubProjeto';
    const dataSourceProductsProdExterno = 'select_products_prodexterno';
    
    protected $parameterBagProjetos          = array("CDPROJETO", "NRORG", "IDPROJETO", "IDSUBPROJETO");
    protected $parameterBagEmails            = array("CDPROJETO", "CDEMAILTEC", "NRORG", "DSEMAILTEC");
    
    protected $parameterBagUnidade           = array("IDPROJETOLOCAL", "CDFILIAL", "CDPROJETO", "NRORG");
    
    protected $parameterBagEmailUnidade      = array("CDPROJETO", "CDFILIAL", "CDEMAILTEC", "DSEMAILTEC", "NRORG");
    protected $parameterBagSubProjetos       = array("CDPROJETO", "CDSUBPROJETO", "IDSUBPROJETO", "DSSUBPROJETO", "NRORG", "IDATIVO");
    protected $parameterBagEmailSubProjeto   = array("CDPROJETO", "CDSUBPROJETO", "NRORG");
    
    protected $parameterBagUnidadeSubProjeto = array("IDPROJETOLOCAL", "CDFILIAL", "CDPROJETO", "NRORG", "CDSUBPROJETO");
    
    protected $parameterBagEmailUnidadeParticipanteSubProjeto = array ("CDPROJETO", "CDSUBPROJETO", "CDFILIAL", "CDEMAILTEC", "DSEMAILTEC", "NRORG");
    
    protected $parameterBagCodExterno                  = array("CDPRODUTO", "CDPRODINTE", "CDPROJETO", "NRORG");
    protected $parameterBagCodExternoUnidade           = array("CDPRODUTO", "CDPRODINTE", "CDPROJETO", "NRORG", "CDFILIAL");
    protected $parameterBagCodExternoSubProjeto        = array("CDPRODUTO", "CDPRODINTE", "CDPROJETO", "CDSUBPROJETO", "NRORG");
    protected $parameterBagCodExternoUnidadeSubProjeto = array("CDPRODUTO", "CDPRODINTE", "CDPROJETO", "CDSUBPROJETO", "NRORG", "CDFILIAL");
    
    protected $parameterBagProductsProdExterno = array("CDPRODUTO", "CDFILIAL", "NMPRODUTO", "CDARVPROD", "CDNVPRODUTO", "NRORG");
    
    public function __construct ($crudService, $parameterBag, $paramBag, Manager $dataSourceManager,  Utilities\Util $util,  $novoCodigo, $ParaEmailRespService) {
        $this->crudService              = $crudService;
        $this->parameterBag             = $parameterBag;
        $this->dataSourceManager        = $dataSourceManager;
        $this->util                     = $util;
        $this->novoCodigo               = $novoCodigo;
        $this->ParaEmailRespService     = $ParaEmailRespService;
    }
    
    public function findProductsProdExterno(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProductsProdExterno;
        $this->parameterBagColumns = $this->parameterBagProductsProdExterno;
        parent::find($request,$response);
    }
    
    //Aba Projetos
    
    public function findPŕojetosDisponiveis(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProjetosDisponiveis;
        $this->parameterBagColumns = $this->parameterBagProjetos;
        parent::find($request,$response);
    }
    
    public function saveProjeto(DTO\Request $request, DTO\Response $response){
        $params = $request->getDataSet()->getRows();
        $this->dataSourceName = self::dataSourceProjetosDisponiveis;
        $this->parameterBagColumns = $this->parameterBagEmails;
        parent::save($request,$response);
    }
    
    // Aba Email (Projeto)
    
    public function findEmails(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmails;
        $this->parameterBagColumns = $this->parameterBagEmails;
        parent::find($request,$response);
    }
    
    public function saveEmailGerente(DTO\Request $request, DTO\Response $response){
        $params = $request->getDataSet()->getRows()[0];
        if($params['__is_new'])
            $params["CDEMAILTEC"] = $this->novoCodigo->getCodeByTableName("LISTAEMAILTEC", $params["NRORG"], 1, 4, false);
        $this->dataSourceName = self::dataSourceEmails;
        $this->parameterBagColumns = $this->parameterBagEmails;
            self::saveEmail($request,$response);
    }
    
    public function saveEmail(DTO\Request $request, DTO\Response $response) {
		try {
			$dataSourceName = $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $rows);
		} catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
	}
	
    public function removeEmail(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmails;
        parent::delete($request, $response);
    }
    
    //Abab Unidade (Projeto)
    
    public function findUnidade(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceUnidade;
        $this->parameterBagColumns = $this->parameterBagUnidade;
        parent::find($request,$response);
    }
    
    public function saveUnidade(DTO\Request $request, DTO\Response $response){
        $conditions = $request->getDataSet()->getRows();
        $this->dataSourceName = self::dataSourceUnidade;
        $this->parameterBagColumns = $this->parameterBagUnidade;
        $conditions[0]['IDPROJETOLOCAL'] = $conditions[0]['CDPROJETO'].$conditions[0]['CDFILIAL'].'000';
        
        $newRequest = new DTO\Request\DataSet(
            new DataSet(
                $this->dataSourceName,
                $conditions
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        $dataSet = $newRequest->getDataSet();
        $dataSet->setDataSourceName($this->dataSourceName);
        parent::save($newRequest, $response);
    }
	
    public function removeUnidade(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceUnidade;
        $rows = $request->getDataSet()->getRows();
        
        foreach($rows as $key => $value){
            $params = array();
            $params[':CDFILIAL'] = $value['CDFILIAL'];
            $params[':NRORG'] = $value['NRORG'];
            $params[':CDPROJETO'] = $value['CDPROJETO'];
            
            $this->ParaEmailRespService->deleteInfosUnidadeParticipantes($params);
        }
        
        parent::delete($request, $response);
    }
    
    // Aba Email Resposnsavel Tecnico da Unidade (Projeto)
    
    public function findEmailsUnidade(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmailUnidade;
        $this->parameterBagColumns = $this->parameterBagEmailUnidade;
        parent::find($request,$response);
    }
    
    public function saveEmailGerenteUnidade(DTO\Request $request, DTO\Response $response){
        $params = $request->getDataSet()->getRows()[0];
        if($params['__is_new'])
            $params["CDEMAILTEC"] = $this->novoCodigo->getCodeByTableName("LISTAEMAILTECLOCAL", $params["NRORG"], 1, 4, false);
        $this->dataSourceName = self::dataSourceEmailUnidade;
        $this->parameterBagColumns = $this->parameterBagEmailUnidade;
            self::saveEmailUnidade($request,$response);
    }
    
    public function saveEmailUnidade(DTO\Request $request, DTO\Response $response) {
		try {
			$dataSourceName = $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $rows);
		} catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
	}
	
    public function removeEmailUnidade(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmailUnidade;
        parent::delete($request, $response);
    }
    
    //Aba SubProjetos
    
    public function findSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceSubProjetos;
        $this->parameterBagColumns = $this->parameterBagSubProjetos;
        parent::find($request,$response);
    }
    
    public function saveSubProjeto(DTO\Request $request, DTO\Response $response){
        $conditions = $request->getDataSet()->getRows();
        $this->dataSourceName = self::dataSourceSubProjetos;
        $conditions[0]['CDSUBPROJETO'] = $this->ParaEmailRespService->novoCodigo($conditions);
        
        $newRequest = new DTO\Request\DataSet(
            new DataSet(
                $this->dataSourceName,
                $conditions
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        $dataSet = $newRequest->getDataSet();
        $dataSet->setDataSourceName($this->dataSourceName);
        parent::save($newRequest, $response);
    }
    
    public function removeSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceSubProjetos;
        $rows = $request->getDataSet()->getRows();
        
        foreach($rows as $key => $value){
            $params = array();
            $params[':NRORG'] = $value['NRORG'];
            $params[':CDPROJETO'] = $value['CDPROJETO'];
            $params[':CDSUBPROJETO'] = $value['CDSUBPROJETO'];
            
            $this->ParaEmailRespService->deleteInfosSubProjeto($params);
        }
        parent::delete($request, $response);
    }
    
    // Aba e-mail (SubProjeto)
    
    public function findEmailsSubPojetos(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmailSubProjeto;
        $this->parameterBagColumns = $this->parameterBagEmailSubProjeto;
        parent::find($request,$response);
    }
    
    public function saveEmailRespTecSubProjeto(DTO\Request $request, DTO\Response $response){
        $params = $request->getDataSet()->getRows()[0];
        if($params['__is_new'])
            $params["CDEMAILTEC"] = $this->novoCodigo->getCodeByTableName("LISTAEMAILTEC", $params["NRORG"], 1, 4, false);
        $this->dataSourceName = self::dataSourceEmailSubProjeto;
        $this->parameterBagColumns = $this->parameterBagEmailSubProjeto;
            self::saveEmailSubProjeto($request,$response);
    }
    
    public function saveEmailSubProjeto(DTO\Request $request, DTO\Response $response) {
		try {
			$dataSourceName = $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $rows);
		} catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
	}
	
    public function removeEmailSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmailSubProjeto;
        parent::delete($request, $response);
    }
    
    // Aba Unidade (SubProjeto)
    
    public function findUnidadeSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceUnidadeSubProjeto;
        $this->parameterBagColumns = $this->parameterBagUnidadeSubProjeto;
        parent::find($request,$response);
    }
    
    public function saveUnidadeSubProjeto(DTO\Request $request, DTO\Response $response){
        $conditions = $request->getDataSet()->getRows();
        $this->dataSourceName = self::dataSourceUnidadeSubProjeto;
        $this->parameterBagColumns = $this->parameterBagUnidadeSubProjeto;
        $conditions[0]['IDPROJETOLOCAL'] = $conditions[0]['CDPROJETO'].$conditions[0]['CDFILIAL'].$conditions[0]['CDSUBPROJETO'];
        
        $newRequest = new DTO\Request\DataSet(
            new DataSet(
                $this->dataSourceName,
                $conditions
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        $dataSet = $newRequest->getDataSet();
        $dataSet->setDataSourceName($this->dataSourceName);
        parent::save($newRequest, $response);
    }
	
    public function removeUnidadeSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceUnidadeSubProjeto;
        $rows = $request->getDataSet()->getRows();
        
        foreach($rows as $key => $value){
            $params = array();
            $params[':CDFILIAL']     = $value['CDFILIAL']; 
            $params[':NRORG']        = $value['NRORG'];
            $params[':CDPROJETO']    = $value['CDPROJETO'];
            $params[':CDSUBPROJETO'] = $value['CDSUBPROJETO'];
            
            $this->ParaEmailRespService->deleteInfosUnidadeParticipantesSubProjeto($params);
        }
        
        parent::delete($request, $response);
    }
    
    // Aba Email Resposnsavel Tecnico da Unidade (Subprojetos)
    
    public function findEmailUnidadeParticipanteSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmailUnidadeParticipanteSubProjeto;
        $this->parameterBagColumns = $this->parameterBagEmailUnidadeParticipanteSubProjeto;
        parent::find($request,$response);
    }
    
    public function saveEmailUnidadeParticipanteSubProjeto(DTO\Request $request, DTO\Response $response){
        $params = $request->getDataSet()->getRows()[0];
        if($params['__is_new'])
            $params["CDEMAILTEC"] = $this->novoCodigo->getCodeByTableName("LISTAEMAILTECLOCAL", $params["NRORG"], 1, 4, false);
        $this->dataSourceName = self::dataSourceEmailUnidadeParticipanteSubProjeto;
        $this->parameterBagColumns = $this->parameterBagEmailUnidadeParticipanteSubProjeto;
        self::saveEmailUnidadeSubProjeto($request,$response);
            
    }
    
    public function saveEmailUnidadeSubProjeto(DTO\Request $request, DTO\Response $response) {
		try {
			$dataSourceName = $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $rows);
		} catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
	}
	
    public function removeEmailUnidadeParticipanteSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmailUnidadeParticipanteSubProjeto;
        parent::delete($request, $response);
    }
    
    // Aba Mapeamento de Produto (Código Externo)
    
    public function findCodigoExterno(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCodExterno;
        $this->parameterBagColumns = $this->parameterBagCodExterno;
        parent::find($request,$response);
    }
    
    public function saveCodigoExterno(DTO\Request $request, DTO\Response $response){
        try {
            $params = $request->getDataSet()->getRows();
            $this->dataSourceName = self::dataSourceCodExterno;
            $this->parameterBagColumns = $this->parameterBagCodExterno;
            parent::save($request,$response);
        }  catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    public function removeCodigoExterno(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCodExterno;
        parent::delete($request, $response);
    }
    
    public function findCodigoExternoUnidade(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCodExternoUnidade;
        $this->parameterBagColumns = $this->parameterBagCodExternoUnidade;
        parent::find($request,$response);
    }
    
    public function saveCodigoExternoUnidade(DTO\Request $request, DTO\Response $response){
        try {
            $params = $request->getDataSet()->getRows();
            $this->dataSourceName = self::dataSourceCodExternoUnidade;
            $this->parameterBagColumns = $this->parameterBagCodExternoUnidade;
            parent::save($request,$response);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    public function removeCodigoExternoUnidade(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCodExternoUnidade;
        parent::delete($request, $response);
    }
    
    // Aba Mapeamento de Produto (Código Externo) - SubProjeto
    
    public function findCodigoExternoSubProjeto (DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCodExternoSubProjeto;
        $this->parameterBagColumns = $this->parameterBagCodExternoSubProjeto;
        parent::find($request,$response);
    }
    
    public function saveCodigoExternoSubProjeto(DTO\Request $request, DTO\Response $response){
        try {
            $params = $request->getDataSet()->getRows();
            $this->dataSourceName = self::dataSourceCodExternoSubProjeto;
            $this->parameterBagColumns = $this->parameterBagCodExternoSubProjeto;
            parent::save($request,$response);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    public function removeCodigoExternoSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCodExternoSubProjeto;
        parent::delete($request, $response);
    }
    
    public function findCodigoExternoUnidadeSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCodExternoUnidadeSubProjeto;
        $this->parameterBagColumns = $this->parameterBagCodExternoUnidadeSubProjeto;
        parent::find($request,$response);
    }
    
    public function saveCodigoExternoUnidadeSubProjeto(DTO\Request $request, DTO\Response $response){
        try {
            $params = $request->getDataSet()->getRows();
            $this->dataSourceName = self::dataSourceCodExternoUnidadeSubProjeto;
            $this->parameterBagColumns = $this->parameterBagCodExternoUnidadeSubProjeto;
            parent::save($request,$response);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    public function removeCodigoExternoUnidadeSubProjeto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCodExternoUnidadeSubProjeto;
        parent::delete($request, $response);
    }
    
    // Exportação de Código Externo
    
    public function exportarCodExterno(DTO\Request\Row $request, DTO\Response $response){
        try{
            $conditions = $this->util->getParamsAsArray($request);
            $this->ParaEmailRespService->exportarCod($conditions);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    // Cadsastro Automático de Código Externo
    
    public function cadAutomaticoCodExterno(DTO\Request\Row $request, DTO\Response $response){
        try{
            $conditions = $this->util->getParamsAsArray($request);
            $this->ParaEmailRespService->autoSaveCod($conditions);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    // Exportação de Código Externo Unidade
    
    public function exportarCodExternoUnidade(DTO\Request\Row $request, DTO\Response $response){
        try{
            $conditions = $this->util->getParamsAsArray($request);
            $this->ParaEmailRespService->exportarCodUnidade($conditions);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    // Cadsastro Automático de Código Externo Unidade
    
    public function cadAutomaticoCodExternoUnidade(DTO\Request\Row $request, DTO\Response $response){
        try{
            $conditions = $this->util->getParamsAsArray($request);
            $this->ParaEmailRespService->autoSaveCodUnidade($conditions);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    // Exportação de Código Externo SubProjeto
    
    public function exportarCodExternoSubProjeto(DTO\Request\Row $request, DTO\Response $response){
        try{
            $conditions = $this->util->getParamsAsArray($request);
            $this->ParaEmailRespService->exportarCodSubProjeto($conditions);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    // Cadsastro Automático de Código Externo SubProjeto
    
    public function cadAutomaticoCodExternoSubProjeto(DTO\Request\Row $request, DTO\Response $response){
        try{
            $conditions = $this->util->getParamsAsArray($request);
            $this->ParaEmailRespService->autoSaveCodSubProjeto($conditions);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    // Exportação de Código Externo Unidade SubProjeto
    
    public function exportarCodExternoUnidadeSubProjeto(DTO\Request\Row $request, DTO\Response $response){
        try{
            $conditions = $this->util->getParamsAsArray($request);
            $this->ParaEmailRespService->exportarCodUnidadeSubProjeto($conditions);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
    
    // Cadsastro Automático de Código Externo Unidade SubProjeto
    
    public function cadAutomaticoCodExternoUnidadeSubProjeto(DTO\Request\Row $request, DTO\Response $response){
        try{
            $conditions = $this->util->getParamsAsArray($request);
            $this->ParaEmailRespService->autoSaveCodUnidadeSubProjeto($conditions);
        } catch (\Exception $e) {
		    if($e->getCode() === 0){
		        $response->setError(new Error("Impossivel adicionar. Registro já existe.", 500));
		    } else {
			    $response->setError(new Error($e->getMessage(), 500));
		    }
		}
    }
}