<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class AutoImpostoService {
	
	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
		$this->connection = $entityManager->getConnection();
	}
	
	public function getProductsAlreadyRegistered($params){
		return $this->entityManager->getConnection()->fetchAll(self::GET_PRODUCT_ALREADY_REGISTRED, $params);
	}
	
	public function getProductsAlreadyRegisteredBetween($params){
		return $this->entityManager->getConnection()->fetchAll(self::GET_PRODUCT_ALREADY_REGISTRED_BETWEEN, $params);
	}
	
	public function getProductToRegister($params){
		return $this->entityManager->getConnection()->fetchAll(self::GET_PRODUCT_TO_REGISTER_BETWEEN, $params);
	}
	
	
	public function deleteProdutosImposto($products) {
		try {
			foreach($products as $product) {
				$this->entityManager->getConnection()->executeQuery(self::DELETE_IMPOSTO_PRODUTO, $product);
			}
		} catch(\Exception $e) {
			var_dump($e);
		}
	}

	public function saveProdutosImpostoLista($products, $params, $AlreadyInsert) {
		try {
			foreach($products as $product) {
				$row = array();
				foreach($params as $key => $value) {
					$row[$key] = $value;
				}
				$row['CDPRODUTO'] = $product;
				if(!isset($AlreadyInsert[$row['CDPRODUTO']])){
					if(isset($row['CDIMPOSTO'])){
						if($row["IDINCIDEPISCOF"]==''){
							$row["IDINCIDEPISCOF"]='N';
						}
						if($row['IDTPIMPOSFIS'] == 'F' || $row['IDTPIMPOSFIS']=='I' || $row['IDTPIMPOSFIS']=='N'){
							$row['VRPEALIMPFIS']=0;
						}
					$this->entityManager->getConnection()->executeQuery(self::INSERE_IMPOSTO_PRODUTO, $row);
					}
				}
			}
		

		} catch(\Exception $e) {
	     	var_dump($e->getMessage());
		}
	}
	

	public function updateProductAliqAuto($product) {
		try {
			if($product['IDTPIMPOSFIS'] == 'F' || $product['IDTPIMPOSFIS']=='I' || $product['IDTPIMPOSFIS']=='N'){
				$product['VRPEALIMPFIS']=0;
			}
			$params = array(
				':VRPEALIMPFIS'  => $product['VRPEALIMPFIS'],
				':VRPERCREDUCAO'  => $product['VRPERCREDUCAO'],
				':IDMODALBASECALC'  => $product['IDMODALBASECALC'],
				':IDINCIDEPISCOF'  => $product['IDINCIDEPISCOF'],
				':VRALIQIBPT'  => $product['VRALIQIBPT'],
				':VRALIQIBPTES'  => $product['VRALIQIBPTES'],
				':CDCFOPPFIS'  => $product['CDCFOPPFIS'],
				':CDCSTICMS'  => $product['CDCSTICMS'],
				':VRALIQPIS'  => $product['VRALIQPIS'],
				':VRALIQCOFINS'  => $product['VRALIQCOFINS'],
				':CDCEST'  => $product['CDCEST'],
				':CDIMPOSTO'  => $product['CDIMPOSTO'],
				':CDCSTPISCOF'  => $product['CDCSTPISCOF'],
				':IDTPIMPOSFIS'  => $product['IDTPIMPOSFIS'],
				':CDFILIAL'  => $product['CDFILIAL'],
				':NRORG'  => $product['NRORG'],
				':CDPRODUTO'  => $product['CDPRODUTO'],
				':VRALIQFCP' => $product['VRALIQFCP'],
				':VRPEALIMPFISEF'=> $product['VRPEALIMPFISEF'],
				':VRPERCREDUCAOEF'=> $product['VRPERCREDUCAOEF'],
				':CDCBENEF'=> $product['CDCBENEF'],
				':VRALIQICMSDES'=> $product['VRALIQICMSDES']
			);
			
			$this->entityManager->getConnection()->executeQuery(self::UPDATE_IMPOSTO_PRODUTO_AUTO, $params);
		} catch(\Exception $e) {
			var_dump($e);
		}
	}
	
	public function saveImpostoProduto($dados, $cdfilial){
		/**
		 * 1. Verifica se o produto existe
		 * 2. Busca se ha informacao do imposto do produto, se sim, atualiza, senao insere
		*/
		$retorno = array(
			'status' => true,
			'rowsSucess' => 0,
			'rowsError' => 0,
			'logRowsError' => '',
			'logRowsSucess' => ''
		);
		
		if(!empty($dados)){
			foreach($dados as $key => $value){
				$row = $this->buildParams($value, $cdfilial);
				$validaRow = $this->validaRow($row);
				if($validaRow['status']){
					$getProduto = $this->getProduto($row);
					if(!empty($getProduto)){
						$codReferencial = $row[':CDPRODUTO'];
						$row[':CDPRODUTO'] = $getProduto;
						$verificaImpostoProduto = $this->getImpostoProduto($row);
						try{
							self::controlTransaction('begin');
							if($verificaImpostoProduto){
								$this->entityManager->getConnection()->executeQuery(self::UPDATE_IMPOSTO_PRODUTO,$row);
								$retorno['rowsSucess'] += 1;
								$retorno['logRowsSucess'] .= 'Imposto do Produto: '. $codReferencial. ' - '. $value['PRODUTO']. ' => ATUALIZADO COM SUCESSO!' . PHP_EOL;	
							}else{
								//insere
								$this->entityManager->getConnection()->executeQuery(self::INSERE_IMPOSTO_PRODUTO,$row);
								$retorno['rowsSucess'] += 1;
								$retorno['logRowsSucess'] .= 'Imposto do Produto: '. $codReferencial. ' - '. $value['PRODUTO']. ' => INSERIDO COM SUCESSO!' . PHP_EOL;	
							}
							self::controlTransaction('commit');
						}catch(\Exception $e){
							self::controlTransaction('rollback');
							$retorno['rowsError'] += 1;
							$retorno['logRowsError'] .= 'Imposto do Produto: '. $codReferencial. ' - '. $value['PRODUTO']. ' => NÃO FOI INSERIDO!' . PHP_EOL . "Erro ao executar operação de banco! Verifique se os valores estão corretamente preenchidos." . PHP_EOL;	
						}
						
					}else{
						$retorno['rowsError'] += 1;
						$retorno['logRowsError'] .= 'Imposto do Produto: '. $row[':CDPRODUTO']. ' - '. $value['PRODUTO']. ' => NÃO FOI INSERIDO!' . PHP_EOL . "Produto não encontrado na base! VErifique se o produto está cadastrado no sistema." . PHP_EOL;	
					}
				}else{
					$retorno['rowsError'] += 1;
					$retorno['logRowsError'] .= 'Imposto do Produto: '. $row[':CDPRODUTO']. ' - '. $value['PRODUTO']. ' => NÃO FOI INSERIDO!' . PHP_EOL . $validaRow['message'] . PHP_EOL;	
				}
			}
		} else{
			$retorno['status'] = false;
			$retorno['message'] = "Não há dados para inserir ou atualizar!";
		}
		
		return $retorno;
	}
	
	public function buildParams($row, $cdfilial){
		$params = array();
        $params[':VRALIQPIS'] = array_key_exists('ALIQPIS', $row) ? $this->valorBD($row['ALIQPIS'])  : null; 
        $params[':VRALIQCOFINS'] = array_key_exists('ALIQCOFINS', $row) ? $this->valorBD($row['ALIQCOFINS']) : null;
        $params[':CDCEST'] = array_key_exists('CEST', $row) ? $row['CEST'] : null;  
        $params[':IDMODALBASECALC'] = array_key_exists('MODBSCALCICMS', $row) ? $row['MODBSCALCICMS'] : null; 
        $params[':VRPERCREDUCAO'] = array_key_exists('PERCREDICMS', $row) ? $this->valorBD($row['PERCREDICMS']) : null; 
        $params[':VRALIQIBPTES'] = array_key_exists('ALIQIBPTE', $row) ? $this->valorBD($row['ALIQIBPTE']) : null; 
        $params[':CDCFOPPFIS'] = array_key_exists('CDCFOP', $row) ? $row['CDCFOP']  : null; 
        $params[':CDCSTPISCOF'] = array_key_exists('CDCSTPISCOFINS', $row) ? $row['CDCSTPISCOFINS'] : null; 
        $params[':CDCSTICMS'] = array_key_exists('CDCSTCSOSNICMS', $row) ? $row['CDCSTCSOSNICMS'] : null; 
        $params[':VRALIQIBPT'] = array_key_exists('ALIQIBPTF', $row)  ? $this->valorBD($row['ALIQIBPTF']) : null; 
        $params[':IDINCIDEPISCOF'] = array_key_exists('INCPISCOFINS', $row)  ? $row['INCPISCOFINS'] : null; 
        $params[':VRPEALIMPFIS'] = array_key_exists('ALIQUOTA', $row) ? $this->valorBD($row['ALIQUOTA']) : null; 
        $params[':CDIMPOSTO'] = array_key_exists('IMPOSTO', $row) ? $row['IMPOSTO'] : null;  
        $params[':CDPRODUTO'] = array_key_exists('CODIGO', $row) ? $row['CODIGO'] : null;  
        $params[':VRALIQFCP'] = array_key_exists('ALIQFCP', $row) ? $this->valorBD($row['ALIQFCP']) : 0;  
        $params[':VRPEALIMPFISEF'] = array_key_exists('ALIQICMS', $row) ? $this->valorBD($row['ALIQICMS']) : 0;  
        $params[':VRPERCREDUCAOEF'] = array_key_exists('PERCREDUCAO', $row) ? $this->valorBD($row['PERCREDUCAO']) : 0;  
        $params[':CDCBENEF'] = array_key_exists('CODBENEFICIOFISCAL', $row) ? $row['CODBENEFICIOFISCAL'] : null;  
        $params[':VRALIQICMSDES'] = array_key_exists('ALIQICMSDES', $row) ? $this->valorBD($row['ALIQICMSDES']) : 0;  
        $params[':CDFILIAL']  = $cdfilial;  
        $params[':DTINCLUSAO']  = date('d/m/Y H:i:s');  
        $params[':DTULTATU']  = date('d/m/Y H:i:s');    
        $params[':NRORGINCLUSAO']  = $this->getNRORG();  
        $params[':NRORGULTATU']  = $this->getNRORG();  
        $params[':IDATIVO']  = 'S';  
        $params[':NRORG']  = $this->getNRORG();  
        $params[':IDTPIMPOSFIS']  = null;
        
        if(!empty($params[':IDINCIDEPISCOF'])){
        	$params[':IDINCIDEPISCOF'] = $this->checkImposto($this->upperCase($params[':IDINCIDEPISCOF'], true));
        }
        
        if(!empty($params[':CDPRODUTO'])){
        	$params[':CDPRODUTO'] = $this->noSpecialsInteiro($params[':CDPRODUTO'], false);
        }
        
        if(!empty($params[':CDIMPOSTO'])){
        	$params[':CDIMPOSTO'] = $this->upperCase($params[':CDIMPOSTO'], true);
        	$dadosgetImposto = $this->getCDIMPOSTO($params[':CDIMPOSTO']);
        	$params[':CDIMPOSTO'] = $dadosgetImposto['CDIMPOSTO'];
        	$params[':IDTPIMPOSFIS'] = $dadosgetImposto['IDTPIMPOSFIS'];
        }
        
        return $params;
	}
	
	public function getProduto($dados){
		$params[':CDARVPROD'] = $dados[':CDPRODUTO'];
		$params[':NRORG'] = $dados[':NRORG'];
		try {
			$result = $this->entityManager->getConnection()->fetchAssoc(self::GET_PRODUTO, $params);
			
			return isset($result['CDPRODUTO']) ? $result['CDPRODUTO'] : null;
		} catch(\Exception $e) {
			return null;
		}
	}
	
	public function getImpostoProduto($dados){
		$params[':CDFILIAL'] = $dados[':CDFILIAL'];
		$params[':CDPRODUTO'] = $dados[':CDPRODUTO'];
		$params[':NRORG'] = $dados[':NRORG'];
		try {
			$result = $this->entityManager->getConnection()->fetchAll(self::GET_PRODUCT_ALREADY_REGISTRED, $params);
			if(!empty($result[0])){
				return true;
			}else{
				return false;
			}
		} catch(\Exception $e) {
			return null;
		}
	}
	
	public function validaRow($row){
		
		$array = array(
            "status" => true,
            "message" => ''
        );
        
        if(empty($row[':CDFILIAL'])){
        	$array['status'] = false;
        	$array['message'] = "Informação da Filial não foi encontrada! Verifique a padronização correta da caluna no Glossário.";
        	
        	return $array;
        }
        
        if(empty($row[':CDPRODUTO'])){
        	$array['status'] = false;
        	$array['message'] = "Código do produto encontra-se vazia! Verifique a padronização correta da caluna no Glossário.";
        	
        	return $array;
        }
        
        if(empty($row[':CDIMPOSTO'])){
        	$array['status'] = false;
        	$array['message'] = "Informação do Imposto não foi encontrada! Verifique a padronização correta da caluna no Glossário.";
        	
        	return $array;
        }
        
        if(empty($row[':NRORG'])){
        	$array['status'] = false;
        	$array['message'] = "Informação da Organização não foi encontrada! Verifique a padronização correta da caluna no Glossário.";
        	
        	return $array;
        }
        
        if(empty($row[':IDINCIDEPISCOF'])){
        	$array['status'] = false;
        	$array['message'] = "Informação se Incide PIS / Cofins não foi encontrada! Verifique a padronização correta da caluna no Glossário.";
        	
        	return $array;
        }
        
        if(empty($row[':VRPEALIMPFIS']) && $row[':VRPEALIMPFIS'] != 0){
        	$array['status'] = false;
        	$array['message'] = "Valor de alíquota não foi encontrada! Verifique a padronização correta da caluna no Glossário.";
        	
        	return $array;
        }else{
        	return $array;
        }
        
	}
public function validaObrigatorio($row){
		$array = array(
            "status" => true,
            "message" => ''
        );
        if($row[0]['CDCSTICMS'] == '' && $row[0]['CDCFOPPFIS'] != ''){
        	
        	$array['status'] = false;
        	$array['message'] = "O Código CST/CSOSN (ICMS) não pode ser vazio devido o campo Código CFOP estar preenchido";
        	
        	return $array;
        	
        }
        
         if($row[0]['CDCFOPPFIS'] == '' && $row[0]['CDCSTICMS'] != ''){
        	
        	$array['status'] = false;
        	$array['message'] = "O código CFOP não pode ser vazio devido o campo CST/CSOSN estar preenchido";
        	
        	return $array;
        	
        }
        
        if($row[0]['CDCFOPPFIS']=='5405'||$row[0]['CDCFOPPFIS']=='5656'||$row[0]['CDCFOPPFIS']=='5667'){
        	if($row[0]['CDCSTICMS']!='60' && $row[0]['CDCSTICMS']!='500'){
        	$array['status'] = false;
        	$array['message'] = "O codigo CFOP selecionado so pode ser usado com o CST/CSOSN 60 OU 500";
        	
        	return $array;
        	}
        }
        if($row[0]['CDCSTPISCOF']=='49'||$row[0]['CDCSTPISCOF']=='03'){
        	$result = $this->verificarCaixaNFC($row[0]['CDFILIAL']);
        	if(!$result){
        		$array['status'] = false;
        		$array['message'] = "O código CST (PIS/COFINS) 49 ou 03 so pode ser usado em filiais que possuem caixa tipo SAT.";
        	
        		return $array;
        	}
        }else{
		     return $array;
		 }
        
	}	
	public function replacevirg($newRow){
		$newRow[0]['VRPEALIMPFIS'] = str_replace ("," , "." , $newRow[0]['VRPEALIMPFIS']);
		$newRow[0]['VRPERCREDUCAO'] = str_replace ("," , "." , $newRow[0]['VRPERCREDUCAO']);
		$newRow[0]['VRALIQIBPT'] = str_replace ("," , "." , $newRow[0]['VRALIQIBPT']);
		$newRow[0]['VRALIQIBPTES'] = str_replace ("," , "." , $newRow[0]['VRALIQIBPTES']);
		$newRow[0]['VRALIQFCP'] = str_replace ("," , "." , $newRow[0]['VRALIQFCP']);
		$newRow[0]['VRPEALIMPFISEF'] = str_replace ("," , "." , $newRow[0]['VRPEALIMPFISEF']);
		$newRow[0]['VRPERCREDUCAOEF'] = str_replace ("," , "." , $newRow[0]['VRPERCREDUCAOEF']);
		$newRow[0]['VRALIQICMSDES'] = str_replace ("," , "." , $newRow[0]['VRALIQICMSDES']);
		
		return $newRow[0];
	}
	
	public function verificarCaixaNFC($row){
    	try{
    		$params = array(
         	":CDFILIAL" 		=>	$row
     		);
     		$response = $this->entityManager->getConnection()->fetchAll(self::FIND_CODIGO, $params);
     		
    		return $response;
    		
    	} catch(\Exception $e){
    		throw $e;
    	}
    }
	
	public function checkImposto($value){
		 switch ($value) {
            case 'NAO':
            	return 'N';
                break;
            case 'NÃO':
            	return 'N';
                break;
            case 'SIM':
            	return 'S';
                break;
            case 'S':
            	return 'S';
                break;
            case 'N':
            	return 'N';
                break;
        }
	}
	
	public function getCDIMPOSTO($value){
		$params[':SGIMPOSTO'] = $value;
		$return['CDIMPOSTO'] = '';
		$return['IDTPIMPOSFIS'] = '';
		try {
			$result = $this->entityManager->getConnection()->fetchAll(self::GET_CDIMPOSTO, $params);
			$return['CDIMPOSTO'] = isset($result[0]['CDIMPOSTO']) ? $result[0]['CDIMPOSTO'] : null;
			$return['IDTPIMPOSFIS'] = isset($result[0]['IDTPIMPOSFIS']) ? $result[0]['IDTPIMPOSFIS'] : null;
			
			return $return;
			
		} catch(\Exception $e) {
			return null;
		}
	}
	
	public function getNRORG(){
		try {
			$result = $this->entityManager->getConnection()->fetchAll(self::GET_NRORG);
			return isset($result[0]['NRORG']) ? $result[0]['NRORG'] : null;
		} catch(\Exception $e) {
			return null;
		}
	}
	
	private function upperCase($text, $noSpecials = false){
		return mb_strtoupper($this->noSpecials($text, $noSpecials), 'UTF-8');
	}
	
	private function noSpecials($texto, $noSpecials){
		/* função que gera uma texto limpo:
		- limpa acentos e transforma em letra normal
		- limpa cedilha e transforma em c normal, o mesmo com o ñ
		- transforma espaços em hifen (-)
		- tira caracteres invalidos
		by Micox - elmicox.blogspot.com - www.ievolutionweb.com
		*/
		//desconvertendo do padrão entitie (tipo á para á)
		$texto = trim(html_entity_decode($texto));
		 
		if($noSpecials){
			//tirando os acentos
			$texto= preg_replace('![áàãâä]+!u','a',$texto);
			$texto= preg_replace('![éèêë]+!u','e',$texto);
			$texto= preg_replace('![íìîï]+!u','i',$texto);
			$texto= preg_replace('![óòõôö]+!u','o',$texto);
			$texto= preg_replace('![úùûü]+!u','u',$texto);
			//parte que tira o cedilha e o ñ
			$texto= preg_replace('![ç]+!u','c',$texto);
			$texto= preg_replace('![ñ]+!u','n',$texto);	
		}
		
		//tirando outros caracteres invalidos
		$texto= preg_replace('[^a-z0-9\-]','-',$texto);
		//tirando espaços
		//$texto = str_replace(' ','-',$texto);
		//trocando duplo espaço (hifen) por 1 hifen só
		$texto = str_replace('--','-',$texto);
	
		return strtolower($texto);
	}
	private function noSpecialsInteiro($texto){
		$valor = preg_replace('/\D/', '', $texto);
		return $texto;
	}
	
	private function valorBD($valorAplicado){
		
		$posVirgula = strpos( $valorAplicado, "," );
		$posPont = strpos( $valorAplicado, "." );
		
		if (!$posVirgula === false && !$posPont === false) {
			$valor = str_replace("." , "" , $valorAplicado ); // Primeiro tira os pontos
			$valorAplicado2 = str_replace(',','.',$valor);
		}else if(!$posVirgula === false && $posPont === false){
			$valorAplicado2 = str_replace(',','.',$valorAplicado);
		}else {
			$valorAplicado2 = $valorAplicado;
		}
		
		return $valorAplicado2;
	}
	
	public function controlTransaction($type){
    // controla transação com banco    
        try {
            switch ($type) {
                case 'begin':
                // inicia transação sem auto-commit
                    $this->entityManager->getConnection()->beginTransaction();
                    break;
                case 'commit':
                // salva transação
                    $this->entityManager->getConnection()->commit();
                    break;
                case 'rollback':
                // descarta transação
                    $this->entityManager->getConnection()->rollBack();
                    break;
            }
        } catch (\Exception $e) {
            
        }
    }
    
    // IMPORTAÇÃO IBPT
    
    public function saveAliquotaIBPT($aliquota, $CDFILIAL, &$array){
    	
    	$retorno = array(
			'status' => true,
			'rowsSucess' => 0,
			'rowsError' => 0,
			'logRowsError' => '',
			'logRowsSucess' => ''
		);
    		if(!empty($aliquota)){
    			foreach($aliquota as $IBPT){
		    		$params = array(
		    			':CDCLASFISC' => $IBPT['codigo'],
		    			':VRALIQIBPT' => $IBPT['importadosfederal'],
		    			':VRALIQIBPTES' => $IBPT['estadual'],
		    			':CDFILIAL'	  => $CDFILIAL
		    		);
    					
		    		$produtos = $this->entityManager->getConnection()->fetchAll(self::GET_PRODUTOS_BY_CDCLASFISC, $params);
		    		foreach($produtos as $prod){
		    			if(!array_key_exists($prod['CDPRODUTO'].$IBPT['codigo'], $array['checkDuplicidade'])){
			    			$params[':CDPRODUTO'] = $prod['CDPRODUTO'];
			    			$this->entityManager->getConnection()->executeQuery(self::UPDATE_IBPT_PRODUTO,$params);	
			    			$array['checkDuplicidade'][$prod['CDPRODUTO'].$IBPT['codigo']] = true; 
			    			$retorno['rowsSucess']++;
							$retorno['logRowsSucess'] .= 'IBTP do Produto: '.$prod['NMPRODUTO']. ' - '.$prod['CDPRODUTO']. ' => ATUALIZADO COM SUCESSO!' . PHP_EOL;	
		    			}
		    		}
		    	}
    		} else{
			$retorno['status'] = false;
			$retorno['message'] = "Não há dados para inserir ou atualizar!";
		}
    	return $retorno;
    }
    
    const GET_PRODUTO = "SELECT CDPRODUTO FROM PRODUTO WHERE CDARVPROD = :CDARVPROD AND NRORG = :NRORG ORDER BY CDNVPRODUTO DESC";
    
    const GET_NRORG = "SELECT NRORG FROM PARAMGERAL";
    
    const GET_CDIMPOSTO = "
    	SELECT CDIMPOSTO, IDTPIMPOSFIS FROM IMPOSTO WHERE SGIMPOSTO = :SGIMPOSTO
    ";

    const GET_PRODUCT_ALREADY_REGISTRED = "
    	SELECT A.CDPRODUTO, A.CDFILIAL, A.NRORG, A.CDIMPOSTO,
    		   A.VRPEALIMPFIS, A.VRPERCREDUCAO, A.IDMODALBASECALC,
			   A.IDINCIDEPISCOF, A.VRALIQIBPT, A.VRALIQIBPTES, A.CDCFOPPFIS,
			   A.CDCSTICMS, A.CDCSTPISCOF, A.VRALIQPIS, A.VRALIQCOFINS, A.CDCEST,
			   A.IDTPIMPOSFIS, 'OLD' AS FLAG,
			   A.VRALIQFCP,
			   A.VRPEALIMPFISEF,
			   A.VRPERCREDUCAOEF,
			   A.CDCBENEF,
			   A.VRALIQICMSDES
		  FROM ALIQIMPFIS A 
		 WHERE A.CDFILIAL = :CDFILIAL
		   AND A.NRORG = :NRORG
		   AND INSTR(:CDPRODUTO, A.CDPRODUTO) > 0
    ";

    const GET_PRODUCT_ALREADY_REGISTRED_BETWEEN = "
    	SELECT A.CDPRODUTO, A.CDFILIAL, A.NRORG, A.CDIMPOSTO,
    		   A.VRPEALIMPFIS, A.VRPERCREDUCAO, A.IDMODALBASECALC,
			   A.IDINCIDEPISCOF, A.VRALIQIBPT, A.VRALIQIBPTES, A.CDCFOPPFIS,
			   A.CDCSTICMS, A.CDCSTPISCOF, A.VRALIQPIS, A.VRALIQCOFINS, A.CDCEST,
			   A.IDTPIMPOSFIS, 'OLD' AS FLAG,
			   A.VRALIQFCP,
			   A.VRPEALIMPFISEF,
			   A.VRPERCREDUCAOEF,
			   A.CDCBENEF,
			   A.VRALIQICMSDES
		  FROM ALIQIMPFIS A 
		  JOIN PRODUTO P
		    ON A.CDPRODUTO = P.CDPRODUTO
		   AND A.NRORG = P.NRORG
		 WHERE A.CDFILIAL = :CDFILIAL
		   AND A.NRORG = :NRORG
		   AND P.CDARVPROD BETWEEN :CDARVPRODINI AND :CDARVPRODFIN
    ";

    const GET_PRODUCT_TO_REGISTER_BETWEEN = "
    	SELECT P.CDPRODUTO
		  FROM PRODUTO P
		 WHERE P.NRORG = :NRORG
		   AND P.CDARVPROD BETWEEN :CDARVPRODINI AND :CDARVPRODFIN
		   AND P.CDNVPRODUTO = :CDNVPRODUTO
		   AND P.IDPRODATIVPR = 'S'
    ";

	const DELETE_IMPOSTO_PRODUTO = "
		DELETE
		  FROM ALIQIMPFIS
		 WHERE CDFILIAL   = :CDFILIAL
		   AND NRORG      = :NRORG
		   AND CDIMPOSTO  = :CDIMPOSTO
		   AND CDPRODUTO  = :CDPRODUTO
	";
    
    const INSERE_IMPOSTO_PRODUTO = "    
    	INSERT INTO ALIQIMPFIS 
    	  (VRPEALIMPFIS, VRPERCREDUCAO, IDMODALBASECALC, IDINCIDEPISCOF, VRALIQIBPT, 
    	   VRALIQIBPTES, CDCFOPPFIS, CDCSTICMS, CDCSTPISCOF, VRALIQPIS, VRALIQCOFINS,
    	   CDCEST, CDIMPOSTO, CDFILIAL, NRORG, CDPRODUTO, IDTPIMPOSFIS, VRALIQFCP, VRPEALIMPFISEF, VRPERCREDUCAOEF, CDCBENEF, VRALIQICMSDES)
    	VALUES 
    	  (:VRPEALIMPFIS, :VRPERCREDUCAO, :IDMODALBASECALC, :IDINCIDEPISCOF, :VRALIQIBPT, 
    	   :VRALIQIBPTES, :CDCFOPPFIS, :CDCSTICMS, :CDCSTPISCOF, :VRALIQPIS, :VRALIQCOFINS,
    	   :CDCEST, :CDIMPOSTO, :CDFILIAL, :NRORG, :CDPRODUTO, :IDTPIMPOSFIS, :VRALIQFCP, :VRPEALIMPFISEF, :VRPERCREDUCAOEF, :CDCBENEF, :VRALIQICMSDES)
    ";
    
    const UPDATE_IMPOSTO_PRODUTO = "
    	UPDATE ALIQIMPFIS
           SET VRPEALIMPFIS = :VRPEALIMPFIS,
	           VRPERCREDUCAO = :VRPERCREDUCAO,
	           IDMODALBASECALC = :IDMODALBASECALC,
	           IDINCIDEPISCOF = :IDINCIDEPISCOF,
	           VRALIQIBPT = :VRALIQIBPT,
	           VRALIQIBPTES = :VRALIQIBPTES,
	           CDCFOPPFIS = :CDCFOPPFIS,
	           CDCSTICMS = :CDCSTICMS,
	           VRALIQPIS = :VRALIQPIS,
	           VRALIQCOFINS = :VRALIQCOFINS,
	           CDCEST = :CDCEST,
	           CDIMPOSTO  = :CDIMPOSTO,
	           CDCSTPISCOF = :CDCSTPISCOF,
	           DTULTATU = :DTULTATU,
	           NRORGULTATU = :NRORGULTATU,
			   IDTPIMPOSFIS = :IDTPIMPOSFIS,
			   VRALIQFCP = :VRALIQFCP,
			   VRPEALIMPFISEF = :VRPEALIMPFISEF,
			   VRPERCREDUCAOEF = :VRPERCREDUCAOEF,
			   CDCBENEF = :CDCBENEF,
			   VRALIQICMSDES = :VRALIQICMSDES
         WHERE CDFILIAL   = :CDFILIAL
		   AND NRORG      = :NRORG
		   AND CDPRODUTO  = :CDPRODUTO
    ";
    
    const UPDATE_IMPOSTO_PRODUTO_AUTO = "
    	UPDATE ALIQIMPFIS
           SET VRPEALIMPFIS = :VRPEALIMPFIS,
	           VRPERCREDUCAO = :VRPERCREDUCAO,
	           IDMODALBASECALC = :IDMODALBASECALC,
	           IDINCIDEPISCOF = :IDINCIDEPISCOF,
	           VRALIQIBPT = :VRALIQIBPT,
	           VRALIQIBPTES = :VRALIQIBPTES,
	           CDCFOPPFIS = :CDCFOPPFIS,
	           CDCSTICMS = :CDCSTICMS,
	           VRALIQPIS = :VRALIQPIS,
	           VRALIQCOFINS = :VRALIQCOFINS,
	           CDCEST = :CDCEST,
	           CDIMPOSTO  = :CDIMPOSTO,
	           CDCSTPISCOF = :CDCSTPISCOF,
			   IDTPIMPOSFIS = :IDTPIMPOSFIS,
			   VRALIQFCP = :VRALIQFCP,
			   VRPEALIMPFISEF = :VRPEALIMPFISEF,
			   VRPERCREDUCAOEF = :VRPERCREDUCAOEF,
			   CDCBENEF = :CDCBENEF,
			   VRALIQICMSDES = :VRALIQICMSDES
         WHERE CDFILIAL   = :CDFILIAL
		   AND NRORG      = :NRORG
		   AND CDPRODUTO  = :CDPRODUTO
    ";
    
    const GET_PRODUTOS_BY_CDCLASFISC = "
		SELECT P.CDARVPROD, P.CDPRODUTO, P.NMPRODUTO, P.CDCLASFISC
			FROM PRODUTO P
			JOIN ALIQIMPFIS A ON
				A.CDPRODUTO = P.CDPRODUTO
			WHERE P.CDCLASFISC = :CDCLASFISC
				AND A.CDFILIAL = :CDFILIAL
    ";
    
    const UPDATE_IBPT_PRODUTO = "
    	UPDATE ALIQIMPFIS SET VRALIQIBPT = :VRALIQIBPT, VRALIQIBPTES = :VRALIQIBPTES WHERE CDPRODUTO = :CDPRODUTO AND CDFILIAL = :CDFILIAL
    ";
    
    const FIND_CODIGO = "
	SELECT
	    *
	FROM
	    CAIXA
	WHERE 
	    IDTPEMISSAOFOS IN ('SAT','FNC')
	    AND CDFILIAL = :CDFILIAL
    ";
}