<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities\Query;

class ParaGeraisImportarImpostoService {

  

    // importação concluída
    const SUCCESS_PC = "SUCCESS_IMP";
    // erro na exportação
    const ERROR_PC = "ERROR_IMP";
    
    const IMPORT_CSV = "csv";
    
    const IMPORT_XLS = "xls";
    
    const IMPORT_XLSX = "lsx";
    
    public function __construct(EntityManager $entityManager)  {
        $this->entityManager = $entityManager;
    }
  
    public function importImposto($dataExport){
    // seleciona qual tipo de exportação será realizada
        $status = null;
        $data   = $dataExport['DATA'];
        $dataFilial = array( 
            "CDFILIAL" => $dataExport['CDFILIAL'],
            "NRORG" => $dataExport['NRORG']
            );

        try {
            self::controlTransaction('begin');            
        } catch (Exception $e) {
            return self::ERROR_PC;
        }
        switch ($dataExport['TYPE']) {
            case self::IMPORT_CSV:
                $status = self::importCSV($data,$dataFilial);
                break;

            case self::IMPORT_XLS:
                $status = self::importXLS($data,$dataFilial);
                break;
            
            case self::IMPORT_XLSX:
                $status = self::importXLS($data,$dataFilial);
                break;
        }
        return $status;
    }

    public function controlTransaction($type){
    // controla transação com banco    
        try {
            switch ($type) {
                case 'begin':
                // inicia transação sem auto-commit
                    $this->entityManager->getConnection()->beginTransaction();
                    break;
                case 'commit':
                // salva transação
                    $this->entityManager->getConnection()->commit();
                    break;
                case 'rollback':
                // descarta transação
                    $this->entityManager->getConnection()->rollBack();
                    break;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function importCSV($data,$dataFilial){
        $paramsQuery  = Array();
        foreach($data as $value){
            if(!array_key_exists("nacionalfederal",$value)){
                array_pop($data);
            }
        }
        try {
           foreach ($data as $value) {
             self::formatUpdateAliqCSV($paramsQuery,$value,$dataFilial);
             $this->entityManager->getConnection()->executeQuery(self::UPDATE_ALIQ, $paramsQuery);
           }
           self::controlTransaction('commit');
           return self::SUCCESS_PC;
        } catch (Exception $e) {
            self::controlTransaction('rollback');
            return self::ERROR_PC;
        }
    }
    
    public function importXLS($data,$dataFilial){
        $paramsQuery  = Array();
        //Em definição de padrão
    }
    
     public function formatUpdateAliqCSV(&$paramsQuery, $value, $dataFilial){
        $paramsQuery = array(
            'CDFILIAL'    => $dataFilial['CDFILIAL'],
            'NRORG'    => $dataFilial['NRORG'],
            'VRALIQIBPT'  => floatval ($value['nacionalfederal']),
            'VRALIQIBPTES'  => floatval ($value['estadual']),
            'CDCLASFISC'  => $value['codigo']
            
        );
    }
    
    public function formatUpdateAliqXLS(&$paramsQuery, $value, $dataFilial){
        $paramsQuery = array(
            'CDFILIAL'    => $dataFilial['CDFILIAL'],
            'NRORG'    => $dataFilial['NRORG'],
            'VRALIQIBPT'  => floatval ($value['Tributação federal para nacionais']),
            'VRALIQIBPTES'  => floatval ($value['Tributação estadual']),
            'CDCLASFISC'  => $value['Código']
            
        );
    }
    
    const UPDATE_ALIQ = "
        UPDATE ALIQIMPFIS A
        SET A.VRALIQIBPT  = :VRALIQIBPT,
          A.VRALIQIBPTES  = :VRALIQIBPTES
        WHERE (A.CDFILIAL = :CDFILIAL)
        AND (A.NRORG = :NRORG)
        AND (EXISTS
          (SELECT CDPRODUTO
          FROM PRODUTO
          WHERE CDCLASFISC = :CDCLASFISC
          AND CDPRODUTO    = A.CDPRODUTO
          AND (NRORG = :NRORG)
          ))
    ";

}